(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.Unistore = factory());
}(this, function () { 'use strict';

  var babelHelpers = {};
  babelHelpers.typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
  };

  babelHelpers.classCallCheck = function (instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  };

  babelHelpers;

  // https://github.com/sindresorhus/object-assign

  var hasOwnProperty = Object.prototype.hasOwnProperty;

  function toObject(val) {
    if (val === null || val === undefined) {
      throw new TypeError('Object.assign cannot be called with null or undefined');
    }

    return Object(val);
  }

  var objectAssign = Object.assign || function (target, source) {
    var to = toObject(target);
    var from = void 0;

    for (var s = 1, len = arguments.length; s < len; s++) {
      from = Object(arguments[s]);

      for (var key in from) {
        if (hasOwnProperty.call(from, key)) {
          to[key] = from[key];
        }
      }
    }

    return to;
  };

  function clone(object) {
    return objectAssign({}, object);
  }

  function assertAdapter(adapter) {
    if (!adapter || typeof adapter.fetch !== 'function') {
      return Promise.reject(ReferenceError('store missing adapter with a `fetch` method returning a Promise'));
    }
  }

  var Store = function () {
    function Store() {
      var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      babelHelpers.classCallCheck(this, Store);

      this.primaryKey = opts.primaryKey || 'id';
      this.adapter = opts.adapter;
      this._registry = {};
      this._data = {};
      this._transforms = [[], []];
    }

    Store.prototype.register = function register(type) {
      var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var data = this._data;
      this._registry[type] = opts;
      data[type] = data[type] || {};
      return this;
    };

    Store.prototype.lookup = function lookup(type) {
      return this._registry[type];
    };

    Store.prototype.clear = function clear() {
      var data = this._data;
      for (var type in data) {
        if (data.hasOwnProperty(type)) {
          data[type] = {};
        }
      }
    };

    Store.prototype.get = function get(type, params) {
      var fnc = this._isManyRequest(type, params) ? '_getMany' : '_getByKey';
      return this[fnc](type, params);
    };

    Store.prototype.push = function push(type, records) {
      var fnc = Array.isArray(records) ? '_pushMany' : '_pushOne';
      return this[fnc](type, records);
    };

    Store.prototype.find = function find(type, params) {
      var fnc = this._isManyRequest(type, params) ? '_findMany' : '_findByKey';
      return this[fnc](type, params);
    };

    Store.prototype.fetch = function fetch(type, params) {
      var fnc = this._isManyRequest(type, params) ? '_fetchMany' : '_fetchByKey';
      return this[fnc](type, params);
    };

    Store.prototype.onTransformOne = function onTransformOne(fnc) {
      this._transforms[0].push(fnc);
    };

    Store.prototype.onTransformMany = function onTransformMany(fnc) {
      this._transforms[1].push(fnc);
    };

    Store.prototype.transform = function transform(type, data, isMany) {
      var transforms = this._transforms[~ ~isMany];
      transforms.forEach(function (fnc) {
        data = fnc(type, data);
      });
      return data;
    };

    /**
     * Private
     */

    Store.prototype._isManyRequest = function _isManyRequest(type, params) {
      var typeofParams = typeof params === 'undefined' ? 'undefined' : babelHelpers.typeof(params);
      return typeofParams === 'undefined' || typeofParams === 'object' && !params[this._primaryKeyForType(type)];
    };

    Store.prototype._primaryKeyForType = function _primaryKeyForType(type) {
      var typeOpts = this.lookup(type);
      return typeOpts && typeOpts.primaryKey || this.primaryKey;
    };

    Store.prototype._getByKey = function _getByKey(type, key) {
      var data = this._data[type];
      var record = data && data[key];
      if (record) {
        return clone(record);
      }
    };

    Store.prototype._getMany = function _getMany(type, params) {
      var _this = this;

      var data = this._data[type];
      if (data) {
        var keys = Object.keys(data);
        return keys.length && keys.map(function (key) {
          return _this._getByKey(type, key);
        }) || [];
      }
    };

    Store.prototype._pushOne = function _pushOne(type, record) {
      record = this.transform(type, record);
      if (record) {
        var key = record[this._primaryKeyForType(type)];
        if (typeof key !== 'undefined') {
          key = key.toString();
          this._data[type][key] = record;
          return this._getByKey(type, key);
        }
      }
    };

    Store.prototype._pushMany = function _pushMany(type, records) {
      var _this2 = this;

      records = this.transform(type, records, true);
      if (records) {
        return records.length && records.map(function (record) {
          return _this2._pushOne(type, record);
        }) || [];
      }
    };

    Store.prototype._findByKey = function _findByKey(type, key) {
      var cached = this._getByKey(type, key);
      return cached && Promise.resolve(cached) || this._fetchByKey(type, key);
    };

    Store.prototype._findMany = function _findMany(type, params) {
      var cached = this._getMany(type, params);
      if (cached && cached.length) {
        return Promise.resolve(cached);
      }
      return this._fetchMany(type, params);
    };

    Store.prototype._fetchByKey = function _fetchByKey(type, key) {
      var _this3 = this;

      return assertAdapter(this.adapter) || this.adapter.fetch(type, key).then(function (data) {
        return _this3._pushOne(type, data);
      });
    };

    Store.prototype._fetchMany = function _fetchMany(type, params) {
      var _this4 = this;

      return assertAdapter(this.adapter) || this.adapter.fetch(type, params).then(function (data) {
        return _this4._pushMany(type, data);
      });
    };

    return Store;
  }();

  return Store;

}));